//here should be the new versions of functions we get called with our hooks

#include "plugin.h"

#define CLIENT_COMMAND_DEFINITIONS
#include "complexmacros.h"
#undef CLIENT_COMMAND_DEFINITIONS

#define DC_UPGRADE		0x38		// 8 + 16 + 32
#define MC_UPGRADE		0x1C0	// 64 + 128 + 256
#define SC_UPGRADE		0xE00	// 512 + 1024 + 2048
#define MASK_ARMOR1		64
#define MASK_JETPACK	512
#define MASK_WEAPONS1	8

#include <sdk_util.h> //useful almost everywhere
#include <usercmd.h>

bool bMsgSetTech;
bool bMsgOverride;
int iMsgCounter;
int gmsgSetTech;

inline void GiveJP(edict_t *player);

int Spawn(edict_t *pEntity) {
	if(FStrEq(STRING(pEntity->v.classname), "worldspawn")) //worldspawn means that everything is just starting up
	{
		//this is the only place you can precache new things, so do it here
		DynamicSizedData.Initialize();
		RETURN_META_VALUE(MRES_HANDLED, 0);
	}

	RETURN_META_VALUE(MRES_IGNORED, 0);
}


//you don't need to really understand this, basically it just uses my macros to 
//reroute client commands to the functions you declare in clientcommands.h
void ClientCommand( edict_t *pEntity )
{
	const char* szUserCommand = CMD_ARGV(0);
	if( szUserCommand[0] == '\0' ) return; //to get rid of warnings if the plugin doesn't make any client commands

	//these next 3 lines will use the super duper DEFINE_CLIENT_COMMAND() macro to route each command to the appropriate function
	#define REGISTERING_CLIENT_COMMANDS
		#include "complexmacros.h"
	#undef REGISTERING_CLIENT_COMMANDS

	RETURN_META(MRES_IGNORED);
}

void CommandStart( const edict_t *player, const struct usercmd_s *cmd, unsigned int random_seed )
{
/*
	// Alien menu items (there are assumptions that these are contiguous)
	ALIEN_EVOLUTION_ONE = 101,		// Carapace
	ALIEN_EVOLUTION_TWO = 102,		// Regeneration
	ALIEN_EVOLUTION_THREE = 103,	// Redemption

	ALIEN_EVOLUTION_SEVEN = 107,	// Celerity
	ALIEN_EVOLUTION_EIGHT = 108,	// Adrenaline
	ALIEN_EVOLUTION_NINE = 109,		// Silence
	ALIEN_EVOLUTION_TEN = 110,		// Cloaking
	ALIEN_EVOLUTION_ELEVEN = 111,	// Focus
	ALIEN_EVOLUTION_TWELVE = 112,	// Scent of fear

  	// Alien lifeforms
	ALIEN_LIFEFORM_ONE = 113,
	ALIEN_LIFEFORM_TWO = 114,
	ALIEN_LIFEFORM_THREE = 115,
	ALIEN_LIFEFORM_FOUR = 116,
	ALIEN_LIFEFORM_FIVE = 117,

	// Unlock alien abilities in Combat mode
	ALIEN_HIVE_TWO_UNLOCK = 118,
	ALIEN_HIVE_THREE_UNLOCK = 126,

	MASK_CARAPACE = 8,		// Alien carapace
	MASK_REGENERATION = 16,		// Alien regeneration
	MASK_REDEMPTION = 32,		// Alien redemption
	
	MASK_CELERITY = 64,		// Alien celerity
	MASK_ADRENALINE = 128,		// Alien adrenaline
	MASK_SILENCE = 256,		// Alien silence
	
	MASK_CLOAKING = 512,		// Alien cloaking
	MASK_FOCUS = 1024,		// Alien focus
	MASK_SCENTOFFEAR = 2048,	// Alien scent of fear
*/
	usercmd_s *g_cmd = (usercmd_s *)cmd;
	switch (g_cmd->impulse)
	{
	case 101:
	case 102:
	case 103:
		if (player->v.iuser4 & DC_UPGRADE)
		{
			g_cmd->impulse = 0;
			RETURN_META(MRES_SUPERCEDE);
		}
		break;
	case 107:
	case 108:
	case 109:
		if (player->v.iuser4 & MC_UPGRADE)
		{
			g_cmd->impulse = 0;
			RETURN_META(MRES_SUPERCEDE);
		}
		break;
	case 110:
	case 111:
	case 112:
		if (player->v.iuser4 & SC_UPGRADE)
		{
			g_cmd->impulse = 0;
			RETURN_META(MRES_SUPERCEDE);
		}
		break;
	case 118:
		if (UTIL_NS_GetPlayerPoints(player) < 2)
		{
			g_cmd->impulse = 0;
			RETURN_META(MRES_SUPERCEDE);
		}
		else
		{
			UTIL_NS_MinusPlayerPoints(player, 1);
		}
		break;
	case 31:
		if ((CVAR_GET_FLOAT("mp_resupplymode") == 2) || ((CVAR_GET_FLOAT("mp_resupplymode") == 1) && (UTIL_NS_GetPlayerPoints(player) < 2)))
		{
			g_cmd->impulse = 0;
			RETURN_META(MRES_SUPERCEDE);
		}
		else if(CVAR_GET_FLOAT("mp_resupplymode") == 1)
		{
			UTIL_NS_MinusPlayerPoints(player, 1);
		}
		break;
	case 39:
		if ((player->v.iuser4 & MASK_ARMOR1) && (UTIL_NS_GetPlayerPoints(player) >= 2))
		{
			UTIL_NS_MinusPlayerPoints(player, 2);
			GiveJP((edict_t*)player);
			DynamicSizedData.bHasJP[ENTINDEX((edict_t*)player)] = true;
		}
		break;
	case 64:
		if ((player->v.iuser4 & MASK_WEAPONS1) && (UTIL_NS_GetPlayerPoints(player) >= 2))
		{
			UTIL_NS_MinusPlayerPoints(player, 1);
		}
		else
		{
			g_cmd->impulse = 0;
			RETURN_META(MRES_SUPERCEDE);
		}
	}
	RETURN_META(MRES_HANDLED);
}

void MessageBegin(int msg_dest, int msg_type, const float *pOrigin, edict_t *ed)
{
	SAFE_USER_MSG(gmsgSetTech, "SetTech", -1 );
	if(msg_type == gmsgSetTech)
	{
		bMsgSetTech = true;
		RETURN_META(MRES_HANDLED);
	}
	RETURN_META(MRES_IGNORED);
}

void WriteByte(int iValue)
{
	if(bMsgSetTech)
	{
		if((iValue == 118) || (iValue == 64) || ((CVAR_GET_FLOAT("mp_resupplymode") == 1) && (iValue == 31)))
		{
			bMsgOverride = true;
			RETURN_META(MRES_HANDLED);
		}
	}
	RETURN_META(MRES_IGNORED);
}

void WriteShort(int iValue)
{
	if(bMsgOverride)
	{
		WRITE_SHORT(2);
		RETURN_META(MRES_SUPERCEDE);
	}
	RETURN_META(MRES_IGNORED);
}

void MessageEnd(void)
{
	if(bMsgSetTech)
	{
		bMsgSetTech = false;
		bMsgOverride = false;
		RETURN_META(MRES_HANDLED);
	}
	RETURN_META(MRES_IGNORED);
}

void PlayerPreThink (edict_t *pPlayer)
{
	if (pPlayer->v.team == 0) DynamicSizedData.bHasJP[ENTINDEX(pPlayer)] = 0;
	if (!pPlayer->v.deadflag && DynamicSizedData.bHasJP[ENTINDEX(pPlayer)] && !(pPlayer->v.iuser4 & MASK_JETPACK))
	{
		GiveJP(pPlayer);
	}
	if (UTIL_NS_GetPlayerExperience(pPlayer) > 999) UTIL_NS_SetPlayerExperience(pPlayer, 999);
	RETURN_META(MRES_IGNORED);
}

inline void GiveJP(edict_t *pPlayer)
{
	pPlayer->v.iuser4 |= MASK_JETPACK;
	pPlayer->v.fuser3 = 1000;
	EMIT_SOUND_DYN2(pPlayer, CHAN_STATIC, "items/pickup_jetpack.wav", 1.0f, ATTN_NORM, 0, 100);
}
