//this file exists so that you can paste functions that you need from util.cpp in the actual SDK
//as well as your new custom helper functions you declared in utilfunctions.h

//REMEMBER: to keep your DLL size down, remove functions you don't need from this file


#include "plugin.h"
#include <sdk_util.h>	//pre-defined function definitions saves us a lot of work
#include <cbase.h>

//decided to just fukkit and add all the headers that show up in the hlsdk util.cpp
#include <gamerules.h>
#include <saverestore.h>
#include <time.h>
#include <shake.h>
#include <decals.h>
#include <player.h>
//#include <weapons.h>

#ifdef linux
	#define PLAYER_POINTS_OFFSET	6264
	#define PLAYER_EXP_OFFSET		6256
#elif defined(_WIN32)
	#define PLAYER_POINTS_OFFSET	6376
	#define PLAYER_EXP_OFFSET		6368
#else /* unknown */
	#error "OS unrecognized"
#endif /* unknown */

DLL_GLOBAL const Vector     g_vecZero = Vector(0,0,0);

int gmsgSayText;







//=========================================================
// UTIL_LogPrintf - Prints a logged message to console.
// Preceded by LOG: ( timestamp ) < message >
//=========================================================
void UTIL_LogPrintf( char *fmt, ... )
{
	va_list			argptr;
	static char		string[1024];
	
	va_start ( argptr, fmt );
	vsprintf ( string, fmt, argptr );
	va_end   ( argptr );

	// Print to server console
	ALERT( at_logged, "%s", string );
}

// returns a CBaseEntity pointer to a player by index.  Only returns if the player is spawned and connected
// otherwise returns NULL
// Index is 1 based
CBaseEntity	*UTIL_PlayerByIndex( int playerIndex )
{
	CBaseEntity *pPlayer = NULL;

	if ( playerIndex > 0 && playerIndex <= gpGlobals->maxClients )
	{
		edict_t *pPlayerEdict = INDEXENT( playerIndex );
		if ( pPlayerEdict && !pPlayerEdict->free )
		{
			pPlayer = CBaseEntity::Instance( pPlayerEdict );	
		}
	}
	
	return pPlayer;
}

edict_t	*UTIL_PlayerEdictByIndex( int playerIndex )
{
	edict_t *pPlayer = NULL;

	if ( playerIndex > 0 && playerIndex <= gpGlobals->maxClients )
	{
		pPlayer = INDEXENT( playerIndex );
		if ( pPlayer && !pPlayer->free )
		{
			return pPlayer;
		}
	}
	
	return pPlayer;
}

void UTIL_ChatPrintf( edict_t *pPlayer, char *fmt, ... )
{
	va_list			argptr;
	static char		string[1024];
	
	va_start ( argptr, fmt );
	vsprintf ( string, fmt, argptr );
	va_end   ( argptr );

	// Print to server console
	if (pPlayer == NULL)
	{
		edict_t *pEdict;
		for (int counter = 1;counter <= gpGlobals->maxClients;++counter)
		{
			pEdict = UTIL_PlayerEdictByIndex(counter);
			if (pEdict != NULL)
			{
				UTIL_SayText( string, pEdict);
			}
		}
	}
	else
	{
		UTIL_SayText( string, pPlayer );
	}
}

void UTIL_ConsolePrintf( edict_t *pPlayer, char *fmt, ... )
{
	va_list			argptr;
	static char		string[1024];
	
	va_start ( argptr, fmt );
	vsprintf ( string, fmt, argptr );
	va_end   ( argptr );

	// Print to server console
	if (pPlayer == NULL)
	{
		edict_t *pEdict;
		for (int counter = 1;counter <= gpGlobals->maxClients;++counter)
		{
			pEdict = UTIL_PlayerEdictByIndex(counter);
			if (pEdict != NULL)
			{
				CLIENT_PRINTF( pEdict, print_console, string );
			}
		}
	}
	else
	{
		CLIENT_PRINTF( pPlayer, print_console, string );
	}
}

void UTIL_SayText( const char *pText, CBaseEntity *pEntity )
{
	
	//if ( !pEntity->IsNetClient() ) //this line crashes hudmessage under linux, so i assume it will do the same here but never bothered to test
	//	return;

	SAFE_USER_MSG(gmsgSayText, "SayText", -1 );

	MESSAGE_BEGIN( MSG_ONE, gmsgSayText, NULL, pEntity->edict() );
		WRITE_BYTE( pEntity->entindex() );
		WRITE_STRING( pText );
	MESSAGE_END();
}

void UTIL_SayText( const char *pText, edict_t *pEntity )
{
	
	//if ( !pEntity->IsNetClient() ) //this line crashes hudmessage under linux, so i assume it will do the same here but never bothered to test
	//	return;

	SAFE_USER_MSG(gmsgSayText, "SayText", -1 );

	MESSAGE_BEGIN( MSG_ONE, gmsgSayText, NULL, pEntity );
		WRITE_BYTE( ENTINDEX(pEntity) );
		WRITE_STRING( pText );
	MESSAGE_END();
}

int UTIL_NS_GetPlayerPoints( const edict_t *pPlayer )
{
	int iPoints = *(int*)((char*)(pPlayer->pvPrivateData)+PLAYER_POINTS_OFFSET);
	int iLevel = UTIL_NS_GetPlayerLevel( pPlayer );
	return (iLevel - iPoints - 1);
}

/*
	XP_LEVEL_1	=     0,
	XP_LEVEL_2	=   100,
	XP_LEVEL_3	=   250,
	XP_LEVEL_4	=   450,
	XP_LEVEL_5	=   700,
	XP_LEVEL_6	=  1000,
	XP_LEVEL_7	=  1350,
	XP_LEVEL_8	=  1750,
	XP_LEVEL_9	=  2200,
	XP_LEVEL_10	=  2700,
*/
int UTIL_NS_GetPlayerLevel( const edict_t *pPlayer )
{
/*
	float xp = UTIL_NS_GetPlayerExperience(pPlayer);
	if (xp > 2700) return 10;
	if (xp > 2200) return 9;
	if (xp > 1750) return 8;
	if (xp > 1350) return 7;
	if (xp > 1000) return 6;
	if (xp > 700) return 5;
	if (xp > 450) return 4;
	if (xp > 250) return 3;
	if (xp > 100) return 2;
	return 1;
*/
	return (int)floor(sqrt(UTIL_NS_GetPlayerExperience(pPlayer) / 25 + 2) - 0.5);
}

void UTIL_NS_MinusPlayerPoints( const edict_t *pPlayer, int iValue )
{
	int *pPoints = (int*)((char*)(pPlayer->pvPrivateData)+PLAYER_POINTS_OFFSET);
	*pPoints = max(*pPoints + iValue, 0);
}

float UTIL_NS_GetPlayerExperience( const edict_t *pPlayer )
{
	return *(float*)((char*)(pPlayer->pvPrivateData)+PLAYER_EXP_OFFSET);
}

void UTIL_NS_SetPlayerExperience( const edict_t *pPlayer, float fExp )
{
	*(float*)((char*)(pPlayer->pvPrivateData)+PLAYER_EXP_OFFSET) = fExp;
}
















unsigned short FixedUnsigned16( float value, float scale )
{
	int output;

	output = (int)(value * scale);
	if ( output < 0 )
		output = 0;
	if ( output > 0xFFFF )
		output = 0xFFFF;

	return (unsigned short)output;
}

short FixedSigned16( float value, float scale )
{
	int output;

	output = (int)(value * scale);

	if ( output > 32767 )
		output = 32767;

	if ( output < -32768 )
		output = -32768;

	return (short)output;
}





